@@ -425,6 +425,14 @@ impl SourceDist {
             dist => dist,
         }
     }
+
+    /// Returns the path to the source distribution, if if it's a local distribution.
+    pub fn as_path(&self) -> Option<&Path> {
+        match self {
+            Self::Path(dist) => Some(&dist.path),
+            _ => None,
+        }
+    }
 }
 
 impl Name for RegistryBuiltDist {
